# AC_LANG([CUDA])
# -----------------------------------------------------------------------------
# Define language CUDA with _AC_LANG_ABBREV set to "cuda", _AC_LANG_PREFIX set
# to "CUDA" (i.e. _AC_LANG_PREFIX[]FLAGS equals to 'CUDAFLAGS'), _AC_CC set to
# "CUDAC", and various language-specific Autoconf macros (e.g. AC_LANG_SOURCE)
# copied from C++ language.
#
# If you want to use AC_LANG([CUDA]), AC_LANG_PUSH([CUDA]), and
# AC_LANG_POP([CUDA]) in the "configure.ac", you need to include this file
# explicitly, e.g. "m4_include([m4/acx_cuda.m4])". Another option is two copy
# the following language definitions to the "acinclude.m4". Otherwise, you can
# use the wrappers ACX_LANG_CUDA, ACX_LANG_PUSH_CUDA, and ACX_LANG_POP_CUDA,
# respectively (see below).
#
AC_LANG_DEFINE([CUDA], [cuda], [CUDA], [CUDAC], [C++],
[ac_ext=cu
ac_compile='$CUDAC -c $CUDAFLAGS conftest.$ac_ext >&AS_MESSAGE_LOG_FD'
ac_link='$CUDAC -o conftest$ac_exeext $CUDAFLAGS $LDFLAGS conftest.$ac_ext $LIBS >&AS_MESSAGE_LOG_FD'
ac_compiler_gnu=$acx_cv_cuda_compiler_gnu
])
dnl Avoid mixing macro definitions that are relevant for C or C++ compiler and
dnl might be irrelevant for CUDA:
m4_copy_force([AC_LANG_CONFTEST()], [AC_LANG_CONFTEST(CUDA)])
m4_define([_AC_LANG_NULL_PROGRAM(CUDA)],
[AC_LANG_PROGRAM(
   [__global__ void conftest_foo() { }],
   [conftest_foo<<<1, 1>>>()])])
AC_DEFUN([AC_LANG_COMPILER(CUDA)], [AC_REQUIRE([ACX_PROG_CUDAC])])

# ACX_LANG_CUDA()
# ACX_LANG_PUSH_CUDA()
# ACX_LANG_POP_CUDA()
# -----------------------------------------------------------------------------
# A set of macro wrapping AC_LANG([CUDA]), AC_LANG_PUSH([CUDA]), and
# AC_LANG_POP([CUDA]), respectively, that can be used in the "configure.ac"
# to switch to/from CUDA language without the need to move the language
# definition (see above) to the "acinclude.m4" file.
#
AC_DEFUN([ACX_LANG_CUDA], [AC_LANG([CUDA])])
AC_DEFUN([ACX_LANG_PUSH_CUDA], [AC_LANG_PUSH([CUDA])])
AC_DEFUN([ACX_LANG_POP_CUDA], [AC_LANG_POP([CUDA])])

# ACX_PROG_CUDAC([LIST-OF-COMPILERS = nvcc])
# -----------------------------------------------------------------------------
# Searches for the CUDA Compiler command among the values of the
# blank-separated list LIST-OF-COMPILERS (defaults to a single value "nvcc").
# Declares precious variables CUDAC and CUDAFLAGS to be set to the compiler
# command and the compiler flags, respectively. If the environment variable
# CUDAC is set, values of the list LIST-OF-COMPILERS are ignored.
#
# Standard Autoconf macros like AC_PROG_CC, AC_PROG_FC, etc. do not check
# whether a secondary compiler can actually compile the code (a compiler is
# called secondary if its AC_PROG_* macro is expanded after the corresponding
# AC_PROG_* macro for some other language). Such behaviour makes it difficult
# for the users to make sure that a working compiler was found. Therefore, this
# macro explicitly checks whether the compiler can actually compiler CUDA code.
# The result is either "yes" or "no" and is cached in the
# acx_cv_prog_cudac_works variable.
#
AC_DEFUN_ONCE([ACX_PROG_CUDAC],
  [AC_LANG_PUSH([CUDA])dnl
   AC_ARG_VAR([CUDAC], [CUDA Compiler command])dnl
   AC_ARG_VAR([CUDAFLAGS], [CUDA Compiler flags])dnl
   _AC_ARG_VAR_LDFLAGS()dnl
   _AC_ARG_VAR_LIBS()dnl
   AS_IF([test -z "$CUDAC"],
     [AC_CHECK_PROGS([CUDAC], [m4_default([$1], [nvcc])])])
   _AS_ECHO_LOG([checking for CUDA compiler version])
   set dummy $ac_compile
   ac_compiler=$[2]
   _AC_DO_LIMIT([$ac_compiler --version >&AS_MESSAGE_LOG_FD])
   AC_PROVIDE_IFELSE([_AC_COMPILER_EXEEXT],
     [AC_CACHE_VAL([acx_cv_prog_cudac_works],
        [_AC_COMPILE_IFELSE([_AC_LANG_NULL_PROGRAM],
           [acx_cv_prog_cudac_works=yes], [acx_cv_prog_cudac_works=no])])],
     [m4_expand_once([_AC_COMPILER_EXEEXT])[]dnl
      AC_CACHE_VAL([acx_cv_prog_cudac_works], [acx_cv_prog_cudac_works=yes])])
   m4_expand_once([_AC_COMPILER_OBJEXT])[]dnl
   _AC_LANG_COMPILER_GNU
   acx_test_CUDAFLAGS=${CUDAFLAGS+set}
   acx_save_CUDAFLAGS=$CUDAFLAGS
   AC_CACHE_CHECK([whether $CUDAC accepts -g], [acx_cv_prog_cudac_g],
     [acx_save_cuda_werror_flag=$ac_cuda_werror_flag
      ac_cuda_werror_flag=yes
      acx_cv_prog_cudac_g=no
      CUDAFLAGS='-g'
      _AC_COMPILE_IFELSE([AC_LANG_PROGRAM],
        [acx_cv_prog_cudac_g=yes],
        [CUDAFLAGS=''
         _AC_COMPILE_IFELSE([AC_LANG_PROGRAM],
           [],
           [ac_cuda_werror_flag=$acx_save_cuda_werror_flag
            CUDAFLAGS='-g'
            _AC_COMPILE_IFELSE([AC_LANG_PROGRAM],
              [acx_cv_prog_cudac_g=yes])])])
      ac_cuda_werror_flag=$acx_save_cuda_werror_flag])
   AS_IF(
     [test "$acx_test_CUDAFLAGS" = set],
     [CUDAFLAGS=$acx_save_CUDAFLAGS],
     [test "$acx_cv_prog_cudac_g" = yes],
     [AS_VAR_IF([ac_cv_cuda_compiler_gnu], [yes],
        [CUDAFLAGS='-g -O2'], [CUDAFLAGS='-g'])],
     [AS_VAR_IF([ac_cv_cuda_compiler_gnu], [yes],
        [CUDAFLAGS='-O2'], [CUDAFLAGS=''])])
   AC_LANG_POP([CUDA])])

# ACX_PROG_CUDAC_CXX11([ACTION-IF-SUCCESS = APPEND-CUDAFLAGS],
#                      [ACTION-IF-FAILURE = FAILURE])
# -----------------------------------------------------------------------------
# Finds the CUDA compiler flag needed to enable support for the C++11 standard.
# The result is either "unsupported" or the actual compiler flag required to
# enable the standard, which may be an empty string.
#
# If successful, runs ACTION-IF-SUCCESS (defaults to appending the result to
# CUDAFLAGS), otherwise runs ACTION-IF-FAILURE (defaults to failing with an
# error message).
#
# The flag is cached in the acx_cv_prog_cudac_cxx11.
#
AC_DEFUN([ACX_PROG_CUDAC_CXX11],
  [AC_REQUIRE([ACX_PROG_CUDAC])dnl
   AC_MSG_CHECKING([for $CUDAC option to enable C++11 features])
   AC_CACHE_VAL([acx_cv_prog_cudac_cxx11],
     [acx_cv_prog_cudac_cxx11=unsupported
      AC_LANG_PUSH([CUDA])dnl
      AC_LANG_CONFTEST([AC_LANG_PROGRAM([[namespace test {
  template<typename T>
  class SomeClass {
  public:
    void operator() (T elem) {}
  };
}]],
        [[test::SomeClass<int> some_class = test::SomeClass<int>();
int* dev_nvalid = nullptr;]])])
      acx_save_CUDAFLAGS=$CUDAFLAGS
      for acx_flag in '' '-std=c++11'; do
        CUDAFLAGS="$acx_save_CUDAFLAGS $acx_flag"
        AC_COMPILE_IFELSE([], [acx_cv_prog_cudac_cxx11=$acx_flag])
        test "x$acx_cv_prog_cudac_cxx11" != xunsupported && break
      done
      CUDAFLAGS=$acx_save_CUDAFLAGS
      rm -f conftest.$ac_ext
      AC_LANG_POP([CUDA])])
   AS_IF([test -n "$acx_cv_prog_cudac_cxx11"],
     [AC_MSG_RESULT([$acx_cv_prog_cudac_cxx11])],
     [AC_MSG_RESULT([none needed])])
   AS_VAR_IF([acx_cv_prog_cudac_cxx11], [unsupported],
     [m4_default([$2],
        [AC_MSG_FAILURE([unable to detect CUDA compiler flag needed to dnl
enable support for the C++11 standard])])],
     [m4_default([$1],
        [AS_IF([test -n "$acx_cv_prog_cudac_cxx11"],
           [AS_VAR_APPEND([CUDAFLAGS], [" $acx_cv_prog_cudac_cxx11"])])])])])

# ACX_CUDAC_LIBRARY_LDFLAGS()
# -----------------------------------------------------------------------------
# Determines the linker flags (e.g., -L and -l) for the CUDA runtime libraries
# that are required to successfully link a CUDA program. The result is set to
# these flags and should be included after LIBS when linking.
#
# The macro is intended to be used in those situations when a program contains
# CUDA objects but is linked with a compiler other than CUDA compiler. Note
# that the result does not contain runtime libraries of the host compiler.
#
# The result is cached in the acx_cv_cudac_libs variable.
#
AC_DEFUN([ACX_CUDAC_LIBRARY_LDFLAGS],
  [AC_REQUIRE([ACX_PROG_CUDAC])dnl
   AC_LANG_PUSH([CUDA])
   AC_CACHE_CHECK([for CUDA libraries of $CUDAC], [acx_cv_cudac_libs],
     [acx_cv_cudac_libs=
      AC_LANG_CONFTEST([AC_LANG_PROGRAM])
dnl Get the verbose output and remove the leading '#$ ':
      acx_save_CUDAFLAGS=$CUDAFLAGS
      CUDAFLAGS="$CUDAFLAGS -v"
      acx_cudac_v_output=`eval $ac_link AS_MESSAGE_LOG_FD>&1 2>&1 | dnl
[sed 's/^#\$ *//']`
      rm -f conftest*
      CUDAFLAGS=$acx_save_CUDAFLAGS
dnl Get the value of the LIBRARIES variable, which will be our marker for the
dnl link command:
      acx_cudac_link_marker=`AS_ECHO(["$acx_cudac_v_output"]) | dnl
[sed -n 's/LIBRARIES= *\(.*\)/\1/p']`
dnl Remove all irrelevant lines from the output:
dnl   1) variable declarations;
dnl   2) lines starting with nvlink (contains the marker but is not what we
dnl      need).
      acx_cudac_v_output=`AS_ECHO(["$acx_cudac_v_output"]) | dnl
[sed '/^[^= ][^= ]*=.*/d;/^nvlink.*/d']`
dnl Find the link command(s):
      acx_cudac_link_line=`AS_ECHO(["$acx_cudac_v_output"]) | dnl
grep "$acx_cudac_link_marker" | tr '\n' ' '`
dnl Extract the flags (currently, we take only -l and -L):
      eval "set dummy $acx_cudac_link_line"; shift
      while test $[]@%:@ != 0; do
        AS_CASE([$[]1],
          [-[[lL]]],
          [AS_CASE([$[]2],
             ['' | -*], [],
             [AS_VAR_APPEND([acx_cv_cudac_libs], [" $[]1$[]2"])
              shift])],
          [-[[lL]]*],
          [AS_VAR_APPEND([acx_cv_cudac_libs], [" $[]1"])])
        shift
      done])
   AC_LANG_POP([CUDA])])
